package info.kwarc.lmh.DefIndexer;

import info.kwarc.mmt.MMT;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.solr.client.solrj.SolrServerException;
import org.apache.solr.client.solrj.impl.ConcurrentUpdateSolrServer;
import org.apache.solr.client.solrj.request.AbstractUpdateRequest;
import org.apache.solr.client.solrj.request.ContentStreamUpdateRequest;
import org.apache.solr.common.util.ContentStream;
import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;
import org.kohsuke.args4j.Option;

public class DefIndexer {
    @Option(name="--solr",usage="URL of the SOLR server")
	String solrUrl = "http://localhost:8983/solr";
    
    @Option(name="--mmt",usage="URL of the MMT server")
	String mmtUrl = "http://localhost:8080";

    @Option(name="--stomp", usage="Connect to STOMP server to serve e.g. authoring environments")
    String stompUrl = "http://localhost:61613";
    
	public DefIndexer() {
	}


	final static Pattern uri  = Pattern.compile("http://mathhub.info/([\\w-]+)/([\\w-]+)/([\\w-\\./]+)\\?([\\w-.]+)(\\?([\\w-.]+))?");
	
	class StringContentStream implements ContentStream {
		String content;
		
		public StringContentStream(String content) {
			this.content = content;
		}
		
		public String getName() {
			return "noname";
		}

		public String getSourceInfo() {
			return "mmt";
		}

		public String getContentType() {
			return "application/xml";
		}

		public Long getSize() {
			return new Long(content.length());
		}

		public InputStream getStream() throws IOException {
			return new ByteArrayInputStream(content.getBytes());
		}

		public Reader getReader() throws IOException {
			return new StringReader(content);
		}
		
	}

	public void run(String[] args) {
        CmdLineParser parser = new CmdLineParser(this);

        try {
            parser.parseArgument(args);
        } catch( CmdLineException e ) {
            System.err.println(e.getMessage());
            System.err.println("java DefIndexer [options...] arguments...");
            parser.printUsage(System.err);
            System.err.println();

            System.err.println("  Example: java DefIndexer --solr http://mathhub.info:8983/solr --mmt http://mathhub.info:8080 ");
            return;
        }
        
		//SolrServer solr = new HttpSolrServer(solrUrl);
		ConcurrentUpdateSolrServer solr = new ConcurrentUpdateSolrServer(solrUrl, 10, 5);
		try {
			solr.deleteByQuery( "*:*" );
			solr.commit();
		} catch (SolrServerException e1) {
			e1.printStackTrace();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		MMT mmt = new MMT(mmtUrl);

		for (String theoryURI : mmt.getTheories()) {
			String content = mmt.getPresentation(theoryURI);
			for (String concept : mmt.getTheoryConcepts(theoryURI)) {
				Matcher m = uri.matcher(concept);
				if (!m.matches()) {
					continue;
				}
				
				String repo_group = m.group(1);
				String repo_name = m.group(2);
				String repo = repo_group+"/"+repo_name;
				String omdoc = m.group(3);
				String theory = m.group(4);
				String symb = m.group(6);
				
				try {
					ContentStreamUpdateRequest up = new ContentStreamUpdateRequest("/update/extract");
					up.addContentStream(new StringContentStream(content));
					up.setParam( "literal.id", concept);
					up.setParam( "literal.repository", repo);
					up.setParam( "literal.repository_name", repo_name);
					up.setParam( "literal.repository_group", repo_group);
					
					up.setParam( "literal.omdoc", omdoc);
					up.setParam( "literal.theory", theory);
					up.setParam( "literal.symbol", symb);
					up.setParam( "literal.description", content);
					//up.setParam( "literal.content", content);
					
					up.setAction(AbstractUpdateRequest.ACTION.COMMIT, true, true);
					solr.request(up);
				} catch (IOException e2) {
					// TODO Auto-generated catch block
					e2.printStackTrace();
				} catch (SolrServerException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}
	public static void main(String[] args) {
		new DefIndexer().run(args);
	}
}
